package MIDlets;

import java.io.*;
import java.util.*;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.io.*;
import javax.microedition.rms.*;

import Session.*;

/**
 *  MIDlet CookieFetch uses cookies to track sessions.
 **/
public class CookieFetch extends MIDlet implements CommandListener {
   
  private Display display;

  // Three on-screen command buttons.
  private Command cancelCommand;
  private Command doneCommand;
  private Command fetchCommand;

  // TextField allows the user to input URL.
  private TextField textField;

  // The fetch truncates after 2048 bytes to avoid using
  // too much memory. 
  private int length = 2048;

  public CookieFetch() throws Exception {

    // Initialize display and command objects.
    display = Display.getDisplay(this);
    cancelCommand = new Command("CANCEL", Command.CANCEL, 1);
    doneCommand = new Command("DONE", Command.CANCEL, 1);
    fetchCommand = new Command("FETCH", Command.SCREEN, 1);
  }

  /**
   * Ask for URL in the initial screen.
   **/
  public void startApp() {

    // Form with TextField to ask for URL.
    Form form = new Form("Cookie Fetch");
    textField = new TextField("URL:", "",
                              80, TextField.ANY);
    form.append( textField );

    // Two buttons on the form.
    form.addCommand(cancelCommand);
    form.addCommand(fetchCommand);

    // The CommandListener is this MIDlet itself.
    form.setCommandListener( (CommandListener) this);

    // Display the form on the LCD screen.
    display.setCurrent(form);

  }

  public void pauseApp() {
  }

  public void destroyApp(boolean unconditional) {
  }

  /**
   * Handle events generated by the Commands in the GUI.
   **/
  public void commandAction(Command command, Displayable screen) {

    if (command == cancelCommand) {
      // Quit this MIDlet if "CANCEL" button is pressed.
      destroyApp(false);
      notifyDestroyed();
    } else if (command == doneCommand) {
      // "DONE" is the same as "CANCEL"
      destroyApp(false);
      notifyDestroyed();
    } else if (command == fetchCommand) {
      try {
        // Get the target URL from the TextField.
        String url = "http://" + textField.getString();
        url = url.trim();
        // Get a cookie aware connection.
        HttpConnection conn = (HttpConnection) RMSCookieConnector.open( url );
        TextBox textBox = new TextBox("Results", null,
                                      length, TextField.ANY);
        // Fetch the remote page and display it in textBox.
        textBox.setString( Utils.fetch(conn, length) );
        textBox.addCommand(doneCommand);
        textBox.setCommandListener( (CommandListener) this);
        display.setCurrent(textBox);
        conn.close();
      } catch (Exception e) {
        Utils.showAlert(display);
      }
    }
  }

}
