package MIDlets;

import java.io.*;
import java.util.*;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.io.*;
import javax.microedition.rms.*;

import Session.*;

import org.kxml.*;
import org.kxml.parser.*;
import org.kxml.kdom.*;

/**
 *  MIDlet XMLFetch uses an XML wrapper to track sessions.
 **/
public class XMLFetch extends MIDlet implements CommandListener {
   
  private Display display;

  // Three on-screen command buttons.
  private Command cancelCommand;
  private Command doneCommand;
  private Command fetchCommand;

  // TextField allows the user to input URL.
  private TextField textField;

  // The fetch truncates after 2048 bytes to avoid using
  // too much memory. 
  private int length = 2048;

  public XMLFetch() throws Exception {

    // Initialize display and command objects.
    display = Display.getDisplay(this);
    cancelCommand = new Command("CANCEL", Command.CANCEL, 1);
    doneCommand = new Command("DONE", Command.CANCEL, 1);
    fetchCommand = new Command("FETCH", Command.SCREEN, 1);
  }

  /**
   * Ask for URL in the initial screen.
   **/
  public void startApp() {

    // Form with TextField to ask for URL.
    Form form = new Form("Wrapped XML Fetch");
    textField = new TextField("URL:", "",
                              80, TextField.ANY);
    form.append( textField );

    // Two buttons on the form.
    form.addCommand(cancelCommand);
    form.addCommand(fetchCommand);

    // The CommandListener is this MIDlet itself.
    form.setCommandListener( (CommandListener) this);

    // Display the form on the LCD screen.
    display.setCurrent(form);

  }

  public void pauseApp() {
  }

  public void destroyApp(boolean unconditional) {
  }

  /**
   * Handle events generated by the Commands in the GUI.
   **/
  public void commandAction(Command command, Displayable screen) {

    if (command == cancelCommand) {
      // Quit this MIDlet if "CANCEL" button is pressed.
      destroyApp(false);
      notifyDestroyed();
    } else if (command == doneCommand) {
      // "DONE" is the same as "CANCEL"
      destroyApp(false);
      notifyDestroyed();
    } else if (command == fetchCommand) {
      try {
        // Get the target URL from the TextField.
        String url = "http://" + textField.getString();
        url = url.trim();
        // Wrap session info around an empty XML document.
        String topost = XMLSessionWrapper.wrapDocument("");

        // Get a normal connection.
        HttpConnection conn = (HttpConnection) Connector.open( url );
        // "POST" the remote URL.
        conn.setRequestMethod(HttpConnection.POST);
        // Get output stream.
        DataOutputStream os = conn.openDataOutputStream(); 
        // Write into output stream.
        os.write( topost.getBytes() );
        os.flush();
        // Close outgoing stream.
        os.close();

        // Get the response stream from the server.
        DataInputStream is = conn.openDataInputStream();
        // Unwrap the incoming stream.
        Node e = XMLSessionWrapper.unwrapDocument( is );

        // Close response stream.
        is.close();

        // Get the fetched document in String format:
        String fetched = e.toString().trim();
        // Truncate after "length".
        if ( fetched.length() > length ) {
           fetched = fetched.substring(0, length -1);
        }
 
        // Display fetched page in textBox.
        TextBox textBox = new TextBox("Results", null,
                                      length, TextField.ANY);
        textBox.setString( fetched );
        textBox.addCommand(doneCommand);
        textBox.setCommandListener( (CommandListener) this);
        display.setCurrent(textBox);
        conn.close();
      } catch (Exception e) {
        Utils.showAlert(display);
      }
    }
  }

}
