package Session;

import java.io.*;

import javax.microedition.io.*;

import org.kxml.*;
import org.kxml.parser.*;
import org.kxml.kdom.*;

// ALL WHITE SPACES ARE SIGNIFICANT IN OUR XML DOCUMENTS!
public class XMLSessionWrapper {
    private XMLSessionWrapper() {}

    // This simple solution only uses a simple sessionID.
    // We could add more client state information objects if needed.
    private static String sessionID;

    // Embed sessionID into a wrapper XML document around xmlDoc.
    public static String wrapDocument(String xmlDoc) {
      String wrapID;
      if ( sessionID == null ) {
        wrapID = "";
      } else {
        wrapID = sessionID;
      }
      return "<SessionWrapper><SessionID id=\"" +
             wrapID +
             "\"/>" + xmlDoc + "<OriginalContent>" +
             "</OriginalContent></SessionWrapper>";
    }
    
    // Unwrap an input stream "xmlStream"
    // which contains a sessionID wrapper around the original data.
    // Return a Node representing the unwrapped data.
    public static Node unwrapDocument(InputStream xmlStream) 
                                                throws Exception {
      InputStreamReader reader = new InputStreamReader(xmlStream);
      XmlParser parser = new XmlParser (reader);
      Document doc = new Document();
      doc.parse(parser);

      // Root element of the wrapped XML document.
      Element wrapper = doc.getRootElement();
      // The first element under root contains session info.
      Element sessionInfo = (Element) wrapper.getChild(0);
      // The second element under root is the orginal XMl document.
      Element originalContent = (Element) wrapper.getChild(1);

      // Set static variable sessionID.
      sessionID = sessionInfo.getAttribute("id").getValue();

      // The unwrapped original content is a node under OriginalContent.
      return (Node) originalContent.getChild(0);
    }
}
